#!perl

use lib '.', 't/';
use File::Temp qw/tempfile/;
use Test::More;
use TestNvim;

my $tester = TestNvim->new;
my $vim = $tester->start();

my $uis = $vim->list_uis();

is scalar (@$uis), 0;

done_testing();

